@@ -43,15 +43,36 @@ vcard_from_couch_document (CouchDBDocument *document)
 		return NULL;
 
 	contact = e_contact_new ();
+	e_vcard_add_attribute_with_value (E_VCARD (contact),
+					  e_vcard_attribute_new (NULL, COUCHDB_REVISION_PROP),
+					  couchdb_document_get_revision (document));
+
 	e_contact_set (contact, E_CONTACT_UID, (const gpointer) couchdb_document_get_id (document));
 	e_contact_set (contact, E_CONTACT_GIVEN_NAME,
 		       (const gpointer) couchdb_document_contact_get_first_name (document));
 	e_contact_set (contact, E_CONTACT_FAMILY_NAME,
 		       (const gpointer) couchdb_document_contact_get_last_name (document));
+	e_contact_set (contact, E_CONTACT_NICKNAME,
+		       (const gpointer) couchdb_document_contact_get_nick_name (document));
+	e_contact_set (contact, E_CONTACT_SPOUSE,
+		       (const gpointer) couchdb_document_contact_get_spouse_name (document));
 
-	e_vcard_add_attribute_with_value (E_VCARD (contact),
-					  e_vcard_attribute_new (NULL, COUCHDB_REVISION_PROP),
-					  couchdb_document_get_revision (document));
+	e_contact_set (contact, E_CONTACT_ORG,
+		       (const gpointer) couchdb_document_contact_get_company (document));
+	e_contact_set (contact, E_CONTACT_ORG_UNIT,
+		       (const gpointer) couchdb_document_contact_get_department (document));
+	e_contact_set (contact, E_CONTACT_TITLE,
+		       (const gpointer) couchdb_document_contact_get_title (document));
+	e_contact_set (contact, E_CONTACT_ROLE,
+		       (const gpointer) couchdb_document_contact_get_job_title (document));
+	e_contact_set (contact, E_CONTACT_MANAGER,
+		       (const gpointer) couchdb_document_contact_get_manager_name (document));
+	e_contact_set (contact, E_CONTACT_ASSISTANT,
+		       (const gpointer) couchdb_document_contact_get_assistant_name (document));
+	e_contact_set (contact, E_CONTACT_OFFICE,
+		       (const gpointer) couchdb_document_contact_get_office (document));
+	e_contact_set (contact, E_CONTACT_NOTE,
+		       (const gpointer) couchdb_document_contact_get_notes (document));
 
 	/* parse email addresses */
 	attr_list = NULL;
@@ -199,9 +220,6 @@ vcard_from_couch_document (CouchDBDocument *document)
 									e_vcard_attribute_param_new (EVC_TYPE),
 									EVC_X_COMPANY);
 			} else {
-				//e_vcard_attribute_add_param_with_value (attr,
-				//					e_vcard_attribute_param_new (EVC_TYPE),
-				//					"OTHER");
 				e_vcard_attribute_add_param_with_value (attr,
 									e_vcard_attribute_param_new (EVC_TYPE),
 									"VOICE");
@@ -247,6 +265,40 @@ vcard_from_couch_document (CouchDBDocument *document)
 		e_contact_address_free (contact_address);
 	}
 
+	/* parse URLs */
+	list = couchdb_document_contact_get_urls (document);
+	while (list != NULL) {
+		const char *description_str, *address_str, *uuid_str;
+		EVCardAttribute *attr;
+		CouchDBStructField *url = (CouchDBStructField *) list->data;
+
+		address_str = couchdb_document_contact_url_get_address (url);
+		description_str = couchdb_document_contact_url_get_description (url);
+		uuid_str = couchdb_struct_field_get_uuid (url);
+
+		if (description_str != NULL) {
+			if (g_ascii_strcasecmp (description_str, "blog") == 0)
+				attr = e_vcard_attribute_new (NULL, e_contact_vcard_attribute (E_CONTACT_BLOG_URL));
+			else
+				attr = e_vcard_attribute_new (NULL, e_contact_vcard_attribute (E_CONTACT_HOMEPAGE_URL));
+		} else
+			attr = e_vcard_attribute_new (NULL, e_contact_vcard_attribute (E_CONTACT_HOMEPAGE_URL));
+
+		if (uuid_str != NULL) {
+			EVCardAttributeParam *param;
+
+			param = e_vcard_attribute_param_new (COUCHDB_UUID_PROP);
+			e_vcard_attribute_add_param_with_value (attr, param, uuid_str);
+		}
+
+		e_vcard_attribute_add_value (attr, address_str);
+		e_vcard_add_attribute (E_VCARD (contact), attr);
+
+		/* remove urls from list */
+		list = g_slist_remove (list, url);
+		couchdb_struct_field_unref (url);
+	}
+
 	/* birth date */
 	str = (char *) couchdb_document_contact_get_birth_date (document);
 	if (str) {
@@ -259,6 +311,18 @@ vcard_from_couch_document (CouchDBDocument *document)
 		}
 	}
 
+	/* wedding date */
+	str = (char *) couchdb_document_contact_get_wedding_date (document);
+	if (str) {
+		EContactDate *dt;
+
+		dt = e_contact_date_from_string (str);
+		if (dt) {
+			e_contact_set (contact, E_CONTACT_ANNIVERSARY, (const gpointer) dt);
+			e_contact_date_free (dt);
+		}
+	}
+
 	/* application annotations */
 	if (couchdb_document_has_field (document, "application_annotations")) {
 		CouchDBStructField *annotations = couchdb_document_get_application_annotations (document);
@@ -391,6 +455,34 @@ contact_phone_to_struct_field (EVCardAttribute *attr)
 	return sf;
 }
 
+static CouchDBStructField *
+contact_url_to_struct_field (EVCardAttribute *attr, const gchar *description)
+{
+	const gchar *address, *uuid;
+	GList *params, *pl;
+
+	address = e_vcard_attribute_get_value (attr);
+	if (!address)
+		return NULL;
+
+	params = e_vcard_attribute_get_params (attr);
+	if (!params)
+		return couchdb_document_contact_url_new (NULL, address, description);
+
+	for (pl = params; pl != NULL; pl = pl->next) {
+		GList *v;
+		EVCardAttributeParam *p = pl->data;
+
+		if (!g_strcmp0 (COUCHDB_UUID_PROP, e_vcard_attribute_param_get_name (p)) != 0) {
+			v = e_vcard_attribute_param_get_values (p);
+			if (v && v->data)
+				uuid = (const gchar *) v->data;
+		}
+	}
+
+	return couchdb_document_contact_url_new (uuid, address, description);
+}
+
 static CouchDBDocument *
 couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *contact)
 {
@@ -415,6 +507,17 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 
 	couchdb_document_contact_set_first_name (document, (const char *) e_contact_get_const (contact, E_CONTACT_GIVEN_NAME));
 	couchdb_document_contact_set_last_name (document, (const char *) e_contact_get_const (contact, E_CONTACT_FAMILY_NAME));
+	couchdb_document_contact_set_nick_name (document, (const char *) e_contact_get_const (contact, E_CONTACT_NICKNAME));
+	couchdb_document_contact_set_spouse_name (document, (const char *) e_contact_get_const (contact, E_CONTACT_SPOUSE));
+
+	couchdb_document_contact_set_company (document, (const char *) e_contact_get_const (contact, E_CONTACT_ORG));
+	couchdb_document_contact_set_department (document, (const char *) e_contact_get_const (contact, E_CONTACT_ORG_UNIT));
+	couchdb_document_contact_set_title (document, (const char *) e_contact_get_const (contact, E_CONTACT_TITLE));
+	couchdb_document_contact_set_job_title (document, (const char *) e_contact_get_const (contact, E_CONTACT_ROLE));
+	couchdb_document_contact_set_manager_name (document, (const char *) e_contact_get_const (contact, E_CONTACT_MANAGER));
+	couchdb_document_contact_set_assistant_name (document, (const char *) e_contact_get_const (contact, E_CONTACT_ASSISTANT));
+	couchdb_document_contact_set_office (document, (const char *) e_contact_get_const (contact, E_CONTACT_OFFICE));
+	couchdb_document_contact_set_notes (document, (const char *) e_contact_get_const (contact, E_CONTACT_NOTE));
 
 	/* email addresses */
 	list = NULL;
@@ -510,6 +613,31 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 		g_slist_free (list);
 	}
 
+	/* URLS */
+	list = NULL;
+	attr_list = e_vcard_get_attributes (E_VCARD (contact));
+	for (al = attr_list; al != NULL; al = al->next) {
+		CouchDBStructField *sf = NULL;
+		EVCardAttribute *attr = (EVCardAttribute *) al->data;
+
+		if (g_strcmp0 (e_vcard_attribute_get_name (attr),
+			       e_contact_vcard_attribute (E_CONTACT_HOMEPAGE_URL)) == 0)
+			sf = contact_url_to_struct_field (attr, "home page");
+		else if (g_strcmp0 (e_vcard_attribute_get_name (attr),
+				    e_contact_vcard_attribute (E_CONTACT_BLOG_URL)) == 0)
+			sf = contact_url_to_struct_field (attr, "blog");
+
+		if (sf != NULL)
+			list = g_slist_append (list, sf);
+	}
+
+	if (list != NULL) {
+		couchdb_document_contact_set_urls (document, list);
+
+		g_slist_foreach (list, (GFunc) couchdb_struct_field_unref, NULL);
+		g_slist_free (list);
+	}
+
 	/* birth date */
 	dt = (EContactDate *) e_contact_get_const (contact, E_CONTACT_BIRTH_DATE);
 	if (dt) {
@@ -519,6 +647,15 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 		g_free (dt_str);
 	}
 
+	/* wedding date */
+	dt = (EContactDate *) e_contact_get_const (contact, E_CONTACT_ANNIVERSARY);
+	if (dt) {
+		char *dt_str = e_contact_date_to_string (dt);
+		couchdb_document_contact_set_wedding_date (document, (const char *) dt_str);
+
+		g_free (dt_str);
+	}
+
 	/* application annotations */
 	str = e_vcard_attribute_get_value (e_vcard_get_attribute (E_VCARD (contact), COUCHDB_APPLICATION_ANNOTATIONS_PROP));
 	if (str) {
@@ -586,7 +723,7 @@ e_book_backend_couchdb_load_source (EBookBackend *backend,
 		dbus_g_connection_unref (bus);
 
 		if (success)
-			uri = g_strdup_printf ("http://localhost:%d", port);
+			uri = g_strdup_printf ("http://127.0.0.1:%d", port);
 		else {
 			g_warning ("Couldn't get port for desktopcouch: %s", error->message);
 			g_error_free (error);
@@ -597,7 +734,7 @@ e_book_backend_couchdb_load_source (EBookBackend *backend,
 	} else if (g_strcmp0 (property, "remote") == 0)
 		uri = g_strdup_printf ("http://%s", e_source_get_property (source, "couchdb_remote_server"));
 	else
-		uri = g_strdup ("http://localhost:5984");
+		uri = g_strdup ("http://127.0.0.1:5984");
 
 	if (! (couchdb_backend->couchdb = couchdb_new (uri))) {
 		g_free (uri);
@@ -615,8 +752,6 @@ e_book_backend_couchdb_load_source (EBookBackend *backend,
 		if (error) {
 			g_warning ("Could not get CouchDB database info: %s", error->message);
 			g_error_free (error);
-
-			return GNOME_Evolution_Addressbook_NoSuchBook;
 		}
 
 		if (only_if_exists)
@@ -626,8 +761,12 @@ e_book_backend_couchdb_load_source (EBookBackend *backend,
 		error = NULL;
 		if (!couchdb_create_database (couchdb_backend->couchdb,
 					      couchdb_backend->dbname,
-					      &error))
+					      &error)) {
+			g_warning ("Could not create 'contacts' database: %s", error->message);
+			g_error_free (error);
+
 			return GNOME_Evolution_Addressbook_PermissionDenied;
+		}
 	} else
 		couchdb_database_info_unref (db_info);
 
@@ -929,6 +1068,24 @@ e_book_backend_couchdb_get_supported_fields (EBookBackend *backend,
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_GIVEN_NAME)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_FAMILY_NAME)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_FULL_NAME)));
+	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_NICKNAME)));
+	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_SPOUSE)));
+	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_BIRTH_DATE)));
+	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_ANNIVERSARY)));
+	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_NOTE)));
+
+	/* URLS */
+	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_HOMEPAGE_URL)));
+	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_BLOG_URL)));
+
+	/* Company fields */
+	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_ORG)));
+	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_ORG_UNIT)));
+	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_TITLE)));
+	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_ROLE)));
+	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_MANAGER)));
+	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_ASSISTANT)));
+	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_OFFICE)));
 
 	/* Email addresses */
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_EMAIL_1)));
@@ -957,7 +1114,6 @@ e_book_backend_couchdb_get_supported_fields (EBookBackend *backend,
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_ADDRESS_HOME)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_ADDRESS_WORK)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_ADDRESS_OTHER)));
-	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_BIRTH_DATE)));
 
 	e_data_book_respond_get_supported_fields (book, opid,
 						  GNOME_Evolution_Addressbook_Success, fields);
